%
%  Subprogram 1: produce Hessenberg-triangular form.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Transform Bp^{-1} * Ap * ... * B1^{-1} * A1 into the form 
%     
%     Tp^{-1} * Rp * ... * T1^{-1} * R1, 
%
%  where matrices Ti's are upper triangular for i = 1: p,
%
%  and Ri are full matrices for i = 1: p.   
%        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for k = 1: p
          
    for j = 1: n-1
           
        w = zeros(n-j+1,1);
        bw = w;
        [w,b] = house(B(j:n,j,k));
        bw = b * w;
        if ( b ~= 0 )
            B(j:n,j:n,k) = B(j:n,j:n,k) - bw * ( w' * B(j:n,j:n,k) );    
            B(j+1:n,j,k) = zeros(n-j,1);
            A(j:n,:,k) = A(j:n,:,k) - bw * ( w' * A(j:n,:,k) );
        end %{if}
        if ( flag_zmat == 1 )
            if ( j == 1 )
                Z(:,:,k) = eye(n) - bw * w';             
            else
                Z(j:n,:,k) = Z(j:n,:,k) - bw * ( w' * Z(j:n,:,k) ) ; 
            end    
        end %{if}
                
    end %{j}
    
end %{k}    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Transform Tp^{-1} * Rp * ... T1^{-1} * R1 into new matrix product 
%      
%       Bp^{-1} * Ap * ... * B1^{-1} * A1.
%          
%  where matrices Ai and Bj are upper triangular for i = 2: p, j = 1: p.
%
%  Moreover, A1 is upper Hessenberg.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j = 1: n-1

    for k = 2: p    
        
        for i = n: -1: (j+1)
            
%            [c, s] = givens(A(i-1, j, k), A(i, j, k));
            [G,y]  = planerot( [A(i-1, j, k); A(i, j, k)] );
            c      = G(1,1);
            s      = G(1,2);

            [A(i-1, j: n, k), A(i, j: n, k)] = row_rot( A(i-1, j: n, k), A(i, j: n, k), c, s );
            A(i, j, k) = 0;
            [B(i-1, i-1: n, k), B(i, i-1: n, k)] = row_rot( B(i-1, i-1: n, k), B(i, i-1: n, k), c, s );
            if ( flag_zmat == 1 )
                [Z(i-1,:, k), Z(i,:, k)] = row_rot( Z(i-1,:, k), Z(i,:, k), c, s ); 
            end %{if}    
            
%            [c, s] = givens( B(i, i, k), B(i, i-1, k) );
            [G, y] = planerot( [B(i, i, k); B(i, i-1, k)] );
            c      = G(1,1);
            s      = G(1,2);
            [B(1:i, i-1, k), B(1:i, i, k)] = col_rot( B(1:i, i-1, k), B(1:i, i, k), c, -s );
            B(i, i-1, k) = 0 ;
            if ( k < p )
                [A(:, i-1, k+1), A(:, i, k+1)] = col_rot( A(:, i-1, k+1), A(:, i, k+1), c, -s );
                if ( flag_qmat == 1 )
                    [Q(:, i-1, k+1), Q(:, i, k+1)] = col_rot( Q(:, i-1, k+1), Q(:, i, k+1), c, -s );    
                end    
            else %{ k = p }
                [A(:, i-1, 1), A(:, i, 1)] = col_rot( A(:, i-1, 1), A(:, i, 1), c, -s );   
                if ( flag_qmat == 1 )
                    [Q(:, i-1, 1), Q(:, i, 1)] = col_rot( Q(:, i-1, 1), Q(:, i, 1), c, -s );    
                end    
            end    
            
        end %{i}
          
    end %{k}
    
    
    for i = n: -1: (j+2) 
    
%        [c, s] = givens(A(i-1, j, 1), A(i, j, 1));
        [G, y] = planerot( [A(i-1, j, 1); A(i, j, 1)] );
        c      = G(1,1);
        s      = G(1,2);
        [A(i-1, j: n, 1), A(i, j: n, 1)] = row_rot( A(i-1, j: n, 1), A(i, j: n, 1), c, s );
        A(i, j, 1) = 0;
        [B(i-1, i-1: n, 1), B(i, i-1: n, 1)] = row_rot( B(i-1, i-1: n, 1), B(i, i-1: n, 1), c, s );
        if ( flag_zmat == 1 )
            [Z(i-1,:,1), Z(i,:,1)] = row_rot( Z(i-1,:,1), Z(i,:,1), c, s ); 
        end %{if}    
           
%        [c, s] = givens( B(i, i, 1), B(i, i-1, 1) );
        [G, y] = planerot( [B(i, i, 1); B(i, i-1, 1)] );
        c      = G(1,1);
        s      = G(1,2);
        [B(1:i, i-1, 1), B(1:i, i, 1)] = col_rot( B(1:i, i-1, 1), B(1:i, i, 1), c, -s );
        B(i, i-1, 1) = 0 ;
        if ( p > 1 )
            [A(:, i-1, 2), A(:, i, 2)] = col_rot( A(:, i-1, 2), A(:, i, 2), c, -s );   
            if ( flag_qmat == 1 )
                [Q(:, i-1, 2), Q(:, i, 2)] = col_rot( Q(:, i-1, 2), Q(:, i, 2), c, -s );    
            end
        else % { p = 1 }
            [A(:, i-1, 1), A(:, i, 1)] = col_rot( A(:, i-1, 1), A(:, i, 1), c, -s );   
            if ( flag_qmat == 1 )
                [Q(:, i-1, 1), Q(:, i, 1)] = col_rot( Q(:, i-1, 1), Q(:, i, 1), c, -s );    
            end
        end  % {if}  
     
    end %{i}        
    
end %{j}    

