
function [M, L, Q, Z] = reorder_ew(aa, bb, M, L, Q, Z, p) ; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Subprogram: reorder_infew.m
%             
%  Goal: reoedering the infinite eigenvalues of periodic matrix pairs 
%        
%        {L_j, M_j}, j = 1: p, to right-lower corner of M_1.
%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = size(M(:, :, 1), 2);
ubdd = 1.e-15;
da = abs(aa);   db = abs(bb);
ri = [];
for k = 1: n
    if ( da(k,1) >= ubdd*db(k,1) ) & ( imag(aa(k,1)) == 0 )
        ri = [ri; k];
    elseif ( da(k,1) >= ubdd*db(k,1) ) & ( ( imag(aa(k,1)) ~= 0 ) | ( imag(bb(k,1)) ~= 0 ) )
        ri = [ri; k];
        k = k + 1;
    end    
end    

bottom = n;
for k = size(ri): -1: 1
    ii = ri(k, 1);
    if ( ii < n ) & ( M(ii+1, ii, 1) == 0 )  % One infinite ew. 
        rii = [];
        for i = (ii+1): bottom
            if ( M(i, i-1, 1) == 0 )
                rii = [rii; i];
            end
        end %{i}
        swap_no = size(rii, 1);
        for t = 1: swap_no
            if ( t < swap_no )
                i2 = rii(t+1,1)-1;
            else
                i2 = bottom;
            end 
            i1 = rii(t,1)-1;
            n1 = 1;  n2 = i2-rii(t,1)+1; 
                
            a22 = M(i1: i2, i1: i2, :);  e22 = L(i1: i2, i1: i2, :);
            [a22, e22, q22, z22] = swap_ew(a22, e22, n1, n2, 1, 1, p) ; 
            M(i1: i2, i1: i2, :) = a22;
            L(i1: i2, i1: i2, :) = e22;
            for j = 1: p
                if ( j > 1 )
                    M(i1: i2, i2+1: n, j) = q22(:, :, j) * M(i1: i2, i2+1: n, j);
                    M(1: i1-1, i1: i2, j) = M(1: i1-1, i1: i2, j) * z22(:, :, j-1);
                else
                    M(i1: i2, i2+1: n, 1) = q22(:, :, 1) * M(i1: i2, i2+1: n, 1);
                    M(1: i1-1, i1: i2, 1) = M(1: i1-1, i1: i2, 1) * z22(:, :, p);
                end    
                L(i1: i2, i2+1: n, j) = q22(:, :, j) * L(i1: i2, i2+1: n, j);
                L(1: i1-1, i1: i2, j) = L(1: i1-1, i1: i2, j) * z22(:, :, j);
                Q(i1: i2, :, j) = q22(:, :, j) * Q(i1: i2, :, j);
                Z(:, i1: i2, j) = Z(:, i1: i2, j) * z22(:, :, j);
            end %{j}   
        end %{t}    
        bottom = bottom - 1;
    elseif ( ii < n-1 ) & ( M(ii+1, ii, 1) ~= 0 )  % Two complex conjugate ews.
        rii = [];
        for i = (ii+2): bottom
            if ( M(i, i-1, 1) == 0 )
                rii = [rii; i];
            end
        end %{i}
        swap_no = size(rii, 1);
        for t = 1: swap_no
            if ( t < swap_no )
                i2 = rii(t+1,1)-1;
            else
                i2 = bottom;
            end    
            i1 = rii(t,1) - 2;
            n1 = 2;  n2 = i2-rii(t,1)+1; 
            
            a22 = M(i1: i2, i1: i2, :);  e22 = L(i1: i2, i1: i2, :);
            [a22, e22, q22, z22] = swap_ew(a22, e22, n1, n2, 1, 1, p) ; 
            M(i1: i2, i1: i2, :) = a22;
            L(i1: i2, i1: i2, :) = e22;
            for j = 1: p
                if ( j > 1 )
                    M(i1: i2, i2+1: n, j) = q22(:, :, j) * M(i1: i2, i2+1: n, j);
                    M(1: i1-1, i1: i2, j) = M(1: i1-1, i1: i2, j) * z22(:, :, j-1);
                else
                    M(i1: i2, i2+1: n, 1) = q22(:, :, 1) * M(i1: i2, i2+1: n, 1);
                    M(1: i1-1, i1: i2, 1) = M(1: i1-1, i1: i2, 1) * z22(:, :, p);
                end    
                L(i1: i2, i2+1: n, j) = q22(:, :, j) * L(i1: i2, i2+1: n, j);
                L(1: i1-1, i1: i2, j) = L(1: i1-1, i1: i2, j) * z22(:, :, j);
                Q(i1: i2, :, j) = q22(:, :, j) * Q(i1: i2, :, j);
                Z(:, i1: i2, j) = Z(:, i1: i2, j) * z22(:, :, j);
            end %{j}   
        end %{t} 
        bottom = bottom - 2;
    end %{if}
end %{for k}

