%
%  Real periodic Schur forms to complex periodic Schur form.
%   (Only left-upper 2*2 matrices of Am and Em)
%

function [PPi_a, PPi_b, aaj, bbj, Am, Em, Qm, Zm] = rpqz2cpqz(PPi_a, PPi_b, aaj, bbj, Am, Em, Qm, Zm, p);

n = size(Am(:,:,1),1);
M11 = Em(1:2,1:2,p) \ Am(1:2,1:2,p);
for j = p-1: -1: 1
    M11 = M11 / Em(1:2,1:2,j); 
    M11 = M11 * Am(1:2,1:2,j);
end    
[U,T] = schur(M11, 'complex');  
Zm(:,1:2,p) = Zm(:,1:2,p) * U;
Am(1:2,1:2,1) = Am(1:2,1:2,1) * U;
Em(1:2,1:2,p) = Em(1:2,1:2,p) * U;
for j = p: -1: 1
%    [c,s] = givens(Em(1,1,j), Em(2,1,j));
    [G, y] = planerot( [Em(1,1,j); Em(2,1,j)] );
    c      = G(1,1);
    s      = G(1,2);
    [Em(1,:,j), Em(2,:,j)] = row_rot(Em(1,:,j), Em(2,:,j), c, s);
    Em(2,1,j) = 0;
    [Qm(1,:,j), Qm(2,:,j)] = row_rot(Qm(1,:,j), Qm(2,:,j), c, s);
    [Am(1,:,j), Am(2,:,j)] = row_rot(Am(1,:,j), Am(2,:,j), c, s);
    if ( j > 1 )
%        [c,s] = givens(Am(2,2,j), Am(2,1,j));
        [G, y] = planerot( [Am(2,2,j); Am(2,1,j)] );
        c      = G(1,1);
        s      = G(1,2);
        [Am(1:2,1,j), Am(1:2,2,j)] = col_rot(Am(1:2,1,j), Am(1:2,2,j), c, -s);
        Am(2,1,j) = 0;
        [Em(1:2,1,j-1), Em(1:2,2,j-1)] = col_rot(Em(1:2,1,j-1), Em(1:2,2,j-1), c, -s);
        [Zm(:,1,j-1), Zm(:,2,j-1)] = col_rot(Zm(:,1,j-1), Zm(:,2,j-1), c,- s); 
    end
end    
Am(2,1,1) = 0;

% If necessary, reoedering first two ews.
if ( (PPi_a(1,1) / PPi_b(1,1)) ~= T(1,1) )
%    [c,s] = givens(T(1,2), T(2,2)-T(1,1));
    [G, y] = planerot( [T(1,2); T(2,2)-T(1,1)] );
    c      = G(1,1);
    s      = G(1,2);
    [Am(1:2,1,1), Am(1:2,2,1)] = col_rot(Am(1:2,1,1), Am(1:2,2,1), c, s);
    [Em(1:2,1,p), Em(1:2,2,p)] = col_rot(Em(1:2,1,p), Em(1:2,2,p), c, s);
    [Zm(:,1,p), Zm(:,2,p)] = col_rot(Zm(:,1,p), Zm(:,2,p), c, s);
    for j = p: -1: 1
%        [c,s] = givens(Em(1,1,j), Em(2,1,j));
        [G, y] = planerot( [Em(1,1,j); Em(2,1,j)] );
        c      = G(1,1);
        s      = G(1,2);
        [Em(1,:,j), Em(2,:,j)] = row_rot(Em(1,:,j), Em(2,:,j), c, s);
        Em(2,1,j) = 0;
        [Qm(1,:,j), Qm(2,:,j)] = row_rot(Qm(1,:,j), Qm(2,:,j), c, s);
        [Am(1,:,j), Am(2,:,j)] = row_rot(Am(1,:,j), Am(2,:,j), c, s);
        if ( j > 1 )
%            [c,s] = givens(Am(2,2,j), Am(2,1,j));
            [G, y] = planerot( [Am(2,2,j); Am(2,1,j)] );
            c      = G(1,1);
            s      = G(1,2);
            [Am(1:2,1,j), Am(1:2,2,j)] = col_rot(Am(1:2,1,j), Am(1:2,2,j), c, -s);
            Am(2,1,j) = 0;
            [Em(1:2,1,j-1), Em(1:2,2,j-1)] = col_rot(Em(1:2,1,j-1), Em(1:2,2,j-1), c, -s);
            [Zm(:,1,j-1), Zm(:,2,j-1)] = col_rot(Zm(:,1,j-1), Zm(:,2,j-1), c, -s);        
        end
    end
    Am(2,1,1) = 0;
end  

PPi_a(1,1) = 1;  PPi_b(1,1) = 1;  
PPi_a(2,1) = 1;  PPi_b(2,1) = 1;
for j = 1: p
    PPi_a(1,1) = PPi_a(1,1) * Am(1,1,j);
    PPi_b(1,1) = PPi_b(1,1) * Em(1,1,j);
    PPi_a(2,1) = PPi_a(2,1) * Am(2,2,j);
    PPi_b(2,1) = PPi_b(2,1) * Em(2,2,j);
    aaj(j,1) = Am(1,1,j);  bbj(j,1) = Em(1,1,j); 
    aaj(j,2) = Am(2,2,j);  bbj(j,2) = Em(2,2,j);
end    
