%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Subprogram: swap_ew.m 
%
%  This function will reorder two adjacent eigenvalues.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [A, B, Qmat, Zmat] = swap_ew(A, B, n1, n2, flag_zmat, flag_qmat, p) ; 

n3 = n1 + n2;  
BA = zeros(n3); S = BA;
na = 0; nb = 0;  tol = eps;
flag_zero = 0; flag_inf = 0; 
inf_case = 0;
In1 = eye(n1); In2 = eye(n2); 
n11 = n1 + 1 ;
Z22(:, :, p) = zeros(n3);  Q22(:, :, p) = zeros(n3);
Qmat(:, :, p) = zeros(n3);  Zmat(:, :, p) = zeros(n3);

% To store the orthogonal matrices Z_k and Q_k.  
if ( flag_qmat == 1 )
    for k = 1: p
        Q22(:, :, k) = eye(n3);
    end    
end    
if( flag_zmat == 1 )
    for k = 1: p
        Z22(:, :, k) = eye(n3); 
    end    
end    


if  ( n3 == 3 )
     
     for k = 1: p
         if ( k > 1 )
             err = eps * norm(A(:, :, k), inf); 
             ia = max( find( abs( diag(A(:, :, k)) ) <= err ) );
             if ( isempty(ia) == 0 ) % There is a zero eigenvalue. 
                 A(ia, ia, k) = 0;
                 na = k;
                 flag_zero = 1;
                 break
             end        
         end  
         err = eps * norm(B(:, :, k), inf); 
         ib = max( find( abs( diag(B(:, :, k)) ) <= err ) );
         if ( isempty(ib) == 0 ) % There is an inf eigenvalue. 
             B(ib, ib, k) = 0;
             nb = k;
             flag_inf = 1;
             break
         end    
     end %{k} 
     
 elseif ( n3 == 2 )       
     
     for k = 1: p
%          err = eps * norm(A(:, :, k), inf); 
%          ia = max( find( abs( diag(A(:, :, k)) ) <= err ) );
%          if ( isempty(ia) == 0 ) % There is a zero eigenvalue. 
%              A(ia, ia, k) = 0;
%              na = k;
%              flag_zero = 1;
%              break
%          end        
          
         err = eps * norm(B(:, :, k), inf); 
         ib = max( find( abs( diag(B(:, :, k)) ) <= err ) );
         if ( isempty(ib) == 0 ) % There is an inf eigenvalue. 
             if ( ib == 1 )
                 inf_case = 1;
             else
                 inf_case = 2;
             end    
             %B(ib, ib, k) = 0;
             nb = k;
             flag_inf = 1;
             break
         end    
     end %{k} 
          
end %{if}


if ( flag_inf == 1 )
    
    if ( inf_case == 1 ) 
         
        %disp('  Infinity EW ( Case I ) of 2-by-2 matrices... ')
        for k = nb: -1: 1  
            if ( k < nb )
%                [c, s] = givens(B(1, 1, k), B(2, 1, k));
                [G, y] = planerot( [B(1, 1, k); B(2, 1, k)] );
                c      = G(1,1);
                s      = G(1,2);
                [B(1, 1: n3, k), B(2, 1: n3, k)] = row_rot(B(1, 1: n3, k), B(2, 1: n3, k), c, s);
                %B(2, 1, k) = 0;
            else    
%                [c, s] = givens(B(1, 2, k), B(2, 2, k));
                [G, y] = planerot( [B(1, 2, k); B(2, 2, k)] );
                c      = G(1,1);
                s      = G(1,2);
                [B(1, 2: n3, k), B(2, 2: n3, k)] = row_rot(B(1, 2: n3, k), B(2, 2: n3, k), c, s);
                %B(2, 2, k) = 0;
            end    
            [A(1, 1: n3, k), A(2, 1: n3, k)] = row_rot(A(1, 1: n3, k), A(2, 1: n3, k), c, s);
            if ( flag_zmat == 1 )
                [Z22(1, :, k), Z22(2, :, k)] = row_rot(Z22(1, :, k), Z22(2, :, k), c, s);
            end    
                 
            if ( k > 1 )
%                [c, s] = givens(A(2, 2, k), A(2, 1, k));
                [G, y] = planerot( [A(2, 2, k); A(2, 1, k)] );
                c      = G(1,1);
                s      = G(1,2);
                [A(:, 1, k), A(:, 2, k)] = col_rot(A(:, 1, k), A(:, 2, k), c, -s);
                %A(2, 1, k) = 0;
                [B(:, 1, k-1), B(:, 2, k-1)] = col_rot(B(:, 1, k-1), B(:, 2, k-1), c, -s);
                if ( flag_qmat == 1 )
                    [Q22(:, 1, k), Q22(:, 2, k)] = col_rot(Q22(:, 1, k), Q22(:, 2, k), c, -s);
                end
            end %{if}
        end %{k}
                                                             
%        [c, s] = givens(A(2, 2, 1), A(2, 1, 1));
        [G, y] = planerot( [A(2, 2, 1); A(2, 1, 1)] );
        c      = G(1,1);
        s      = G(1,2);
        [A(:, 1, 1), A(:, 2, 1)] = col_rot(A(:, 1, 1), A(:, 2, 1), c, -s);
        %A(2, 1, 1) = 0;
        [B(:, 1, p), B(:, 2, p)] = col_rot(B(:, 1, p), B(:, 2, p), c, -s);
        if ( flag_qmat == 1 )
            [Q22(:, 1, 1), Q22(:, 2, 1)] = col_rot(Q22(:, 1, 1), Q22(:, 2, 1), c, -s);
        end
           
        for k = p: -1: (nb+1)
%            [c, s] = givens(B(1, 1, k), B(2, 1, k));
            [G, y] = planerot( [B(1, 1, k); B(2, 1, k)] );
            c      = G(1,1);
            s      = G(1,2);
            [B(1, 1: n3, k), B(2, 1: n3, k)] = row_rot(B(1, 1: n3, k), B(2, 1: n3, k), c, s);
            %B(2, 1, k) = 0;
            [A(1, 1: n3, k), A(2, 1: n3, k)] = row_rot(A(1, 1: n3, k), A(2, 1: n3, k), c, s);
            if ( flag_zmat == 1 )
                [Z22(1, :, k), Z22(2, :, k)] = row_rot(Z22(1, :, k), Z22(2, :, k), c, s);
            end    
                         
%            [c, s] = givens(A(2, 2, k), A(2, 1, k));
            [G, y] = planerot( [A(2, 2, k); A(2, 1, k)] );
            c      = G(1,1);
            s      = G(1,2);
            [A(:, 1, k), A(:, 2, k)] = col_rot(A(:, 1, k), A(:, 2, k), c, -s);
            %A(2, 1, k) = 0;
            if ( k > nb+1 )
                [B(:, 1, k-1), B(:, 2, k-1)] = col_rot(B(:, 1, k-1), B(:, 2, k-1), c, -s);
            else    
                [B(1, 1, nb), B(1, 2, nb)] = col_rot(B(1, 1, nb), B(1, 2, nb), c, -s);
            end    
            if ( flag_qmat == 1 )
                [Q22(:, 1, k), Q22(:, 2, k)] = col_rot(Q22(:, 1, k), Q22(:, 2, k), c, -s);
            end
        end %{k}
        
    elseif ( inf_case == 2 ) 
         
        %disp('  Infinity EW ( Case II ) of 2-by-2 matrices... ')
        for k = nb: p  
            if ( k > nb )
%                [c, s] = givens(B(2, 2, k), B(2, 1, k));
                [G, y] = planerot( [B(2, 2, k); B(2, 1, k)] );
                c      = G(1,1);
                s      = G(1,2);
                [B(:, 1, k), B(:, 2, k)] = col_rot(B(:, 1, k), B(:, 2, k), c, -s);
                %B(2, 1, k) = 0;
            else
%                [c, s] = givens(B(1, 2, k), B(1, 1, k));
                [G, y] = planerot( [B(1, 2, k); B(1, 1, k)] );
                c      = G(1,1);
                s      = G(1,2);
                [B(1, 1, k), B(1, 2, k)] = col_rot(B(1, 1, k), B(1, 2, k), c, -s);
                %B(1, 1, k) = 0;
            end    
            if ( k < p )
                [A(:, 1, k+1), A(:, 2, k+1)] = col_rot(A(:, 1, k+1), A(:, 2, k), c, -s);
                if ( flag_qmat == 1 )
                    [Q22(:, 1, k+1), Q22(:, 2, k+1)] = col_rot(Q22(:, 1, k+1), Q22(:, 2, k+1), c, -s);
                end    
            else  % {k = p}
                [A(:, 1, 1), A(:, 2, 1)] = col_rot(A(:, 1, 1), A(:, 2, 1), c, -s);   
                if ( flag_qmat == 1 )
                    [Q22(:, 1, 1), Q22(:, 2, 1)] = col_rot(Q22(:, 1, 1), Q22(:, 2, 1), c, -s);
                end    
            end    
                                
            if ( k < p )
%                [c, s] = givens(A(1, 1, k+1), A(2, 1, k+1));
                [G, y] = planerot( [A(1, 1, k+1); A(2, 1, k+1)] );
                c      = G(1,1);
                s      = G(1,2);
                [A(1, :, k+1), A(2, :, k+1)] = row_rot(A(1, :, k+1), A(2, :, k+1), c, s);
                %A(2, 1, k+1) = 0;
                [B(1, :, k+1), B(2, :, k+1)] = row_rot(B(1, :, k+1), B(2, :, k+1), c, s);
                if ( flag_zmat == 1 )
                    [Z22(1, :, k+1), Z22(2, :, k+1)] = row_rot(Z22(1, :, k+1), Z22(2, :, k+1), c, s);
                end
            end %{if}
        end %{k}
                                                             
%        [c, s] = givens(A(1, 1, 1), A(2, 1, 1));
        [G, y] = planerot( [A(1, 1, 1); A(2, 1, 1)] );
        c      = G(1,1);
        s      = G(1,2);
        [A(1, :, 1), A(2, :, 1)] = row_rot(A(1, :, 1), A(2, :, 1), c, s);
        %A(2, 1, 1) = 0;
        [B(1, :, 1), B(2, :, 1)] = row_rot(B(1, :, 1), B(2, :, 1), c, s);
        if ( flag_zmat == 1 )
            [Z22(1, :, 1), Z22(2, :, 1)] = row_rot(Z22(1, :, 1), Z22(2, :, 1), c, s);
        end
            
        for  k = 1: nb-1
%             [c, s] = givens(B(2, 2, k), B(2, 1, k));
             [G, y] = planerot( [B(2, 2, k); B(2, 1, k)] );
             c      = G(1,1);
             s      = G(1,2);
             [B(:, 1, k), B(:, 2, k)] = col_rot(B(:, 1, k), B(:, 2, k), c, -s);
             %B(2, 1, k) = 0;
             [A(:, 1, k+1), A(:, 2, k+1)] = col_rot(A(:, 1, k+1), A(:, 2, k+1), c, -s);
             if ( flag_qmat == 1 )
                 [Q22(:, 1, k+1), Q22(:, 2, k+1)] = col_rot(Q22(:, 1, k+1), Q22(:, 2, k+1), c, -s);
             end    
                   
%             [c, s] = givens(A(1, 1, k+1), A(2, 1, k+1));
             [G, y] = planerot( [A(1, 1, k+1); A(2, 1, k+1)] );
             c      = G(1,1);
             s      = G(1,2);
             [A(1, :, k+1), A(2, :, k+1)] = row_rot(A(1, :, k+1), A(2, :, k+1), c, s);
             %A(2, 1, k+1) = 0;
             if ( k < nb-1 )
                 [B(1, :, k+1), B(2, :, k+1)] = row_rot(B(1, :, k+1), B(2, :, k+1), c, s);
             else    
                 [B(1, 2, nb), B(2, 2, nb)] = row_rot(B(1, 2, nb), B(2, 2, nb), c, s);
             end    
             if ( flag_zmat == 1 )
                 [Z22(1, :, k+1), Z22(2, :, k+1)] = row_rot(Z22(1, :, k+1), Z22(2, :, k+1), c, s);
             end
        end %{k}
                
    elseif ( n1 == 1 ) & ( n2 == 2 )
        
        %disp('  Infinity EW ( Case I ) ... ')
        for i = n11: n3
            for k = nb: -1: 1  
                if ( k < nb )
%                    [c, s] = givens(B(i-1, i-1, k), B(i, i-1, k));
                    [G, y] = planerot( [B(i-1, i-1, k); B(i, i-1, k)] );
                    c      = G(1,1);
                    s      = G(1,2);
                    [B(i-1, i-1: n3, k), B(i, i-1: n3, k)] = row_rot(B(i-1, i-1: n3, k), B(i, i-1: n3, k), c, s);
                    B(i, i-1, k) = 0;
                else    
%                    [c, s] = givens(B(i-1, i, k), B(i, i, k));
                    [G, y] = planerot( [B(i-1, i, k); B(i, i, k)] );
                    c      = G(1,1);
                    s      = G(1,2);
                    [B(i-1, i: n3, k), B(i, i: n3, k)] = row_rot(B(i-1, i: n3, k), B(i, i: n3, k), c, s);
                    B(i, i, k) = 0;
                end    
                if ( k > 1 )
                    [A(i-1, i-1: n3, k), A(i, i-1: n3, k)] = row_rot(A(i-1, i-1: n3, k), A(i, i-1: n3, k), c, s);
                else % {k = 1}
                    [A(i-1, :, 1), A(i, :, 1)] = row_rot(A(i-1, :, 1), A(i, :, 1), c, s);    
                end    
                if ( flag_zmat == 1 )
                    [Z22(i-1, :, k), Z22(i, :, k)] = row_rot(Z22(i-1, :, k), Z22(i, :, k), c, s);
                end    
                 
                if ( k > 1 )
%                    [c, s] = givens(A(i, i, k), A(i, i-1, k));
                    [G, y] = planerot( [A(i, i, k); A(i, i-1, k)] );
                    c      = G(1,1);
                    s      = G(1,2);
                    [A(i-1: i, i-1, k), A(i-1: i, i, k)] = col_rot(A(i-1: i, i-1, k), A(i-1: i, i, k), c, -s);
                    A(i, i-1, k) = 0;
                    [B(i-1: i, i-1, k-1), B(i-1: i, i, k-1)] = col_rot(B(i-1: i, i-1, k-1), B(i-1: i, i, k-1), c, -s);
                    if ( flag_qmat == 1 )
                        [Q22(:, i-1, k), Q22(:, i, k)] = col_rot(Q22(:, i-1, k), Q22(:, i, k), c, -s);
                    end
                end %{if}
            end %{k}
        end %{i}
                                                     
        for i = n1: n11        
%            [c, s] = givens(A(n3, i+1, 1), A(n3, i, 1));
            [G, y] = planerot( [A(n3, i+1, 1); A(n3, i, 1)] );
            c      = G(1,1);
            s      = G(1,2);
            [A(:, i, 1), A(:, i+1, 1)] = col_rot(A(:, i, 1), A(:, i+1, 1), c, -s);
            A(n3, i, 1) = 0;
            [B(1: i+1, i, p), B(1: i+1, i+1, p)] = col_rot(B(1: i+1, i, p), B(1: i+1, i+1, p), c, -s);
            if ( flag_qmat == 1 )
                [Q22(:, i, 1), Q22(:, i+1, 1)] = col_rot(Q22(:, i, 1), Q22(:, i+1, 1), c, -s);
            end
            
            for  k = p: -1: nb+1
%                 [c, s] = givens(B(i, i, k), B(i+1, i, k));
                 [G, y] = planerot( [B(i, i, k); B(i+1, i, k)] );
                 c      = G(1,1);
                 s      = G(1,2);
                 [B(i, i: n3, k), B(i+1, i: n3, k)] = row_rot(B(i, i: n3, k), B(i+1, i: n3, k), c, s);
                 B(i+1, i, k) = 0;
                 [A(i, i: n3, k), A(i+1, i: n3, k)] = row_rot(A(i, i: n3, k), A(i+1, i: n3, k), c, s);
                 if ( flag_zmat == 1 )
                     [Z22(i, :, k), Z22(i+1, :, k)] = row_rot(Z22(i, :, k), Z22(i+1, :, k), c, s);
                 end    
                         
%                 [c, s] = givens(A(i+1, i+1, k), A(i+1, i, k));
                 [G, y] = planerot( [A(i+1, i+1, k); A(i+1, i, k)] );
                 c      = G(1,1);
                 s      = G(1,2);
                 [A(1: i+1, i, k), A(1: i+1, i+1, k)] = col_rot(A(1: i+1, i, k), A(1: i+1, i+1, k), c, -s);
                 A(i+1, i, k) = 0;
                 if ( k > nb+1 )
                     [B(1: i+1, i, k-1), B(1: i+1, i+1, k-1)] = col_rot(B(1: i+1, i, k-1), B(1: i+1, i+1, k-1), c, -s);
                 else    
                     [B(1: i, i, nb), B(1: i, i+1, nb)] = col_rot(B(1: i, i, nb), B(1: i, i+1, nb), c, -s);
                 end    
                 if ( flag_qmat == 1 )
                     [Q22(:, i, k), Q22(:, i+1, k)] = col_rot(Q22(:, i, k), Q22(:, i+1, k), c, -s);
                 end
            end %{k}
        end %{i}
                    
    elseif ( n1 == 2 ) & ( n2 == 1 ) 
        
        %disp('  Infinity EW ( Case II ) ... ')
        for i = n1: -1: 1
            for k = nb: p  
                if ( k > nb )
%                     [c, s] = givens(B(i+1, i+1, k), B(i+1, i, k));
                     [G, y] = planerot( [B(i+1, i+1, k); B(i+1, i, k)] );
                     c      = G(1,1);
                     s      = G(1,2);
                     [B(1: i+1, i, k), B(1: i+1, i+1, k)] = col_rot(B(1: i+1, i, k), B(1: i+1, i+1, k), c, -s);
                     B(i+1, i, k) = 0;
                 else
%                     [c, s] = givens(B(i, i+1, k), B(i, i, k));
                     [G, y] = planerot( [B(i, i+1, k); B(i, i, k)] );
                     c      = G(1,1);
                     s      = G(1,2);
                     [B(1: i, i, k), B(1: i, i+1, k)] = col_rot(B(1: i, i, k), B(1: i, i+1, k), c, -s);
                     B(i, i, k) = 0;
                 end    
                 if ( k < p )
                     [A(1: i+1, i, k+1), A(1: i+1, i+1, k+1)] = col_rot(A(1: i+1, i, k+1), A(1: i+1, i+1, k), c, -s);
                     if ( flag_qmat == 1 )
                         [Q22(:, i, k+1), Q22(:, i+1, k+1)] = col_rot(Q22(:, i, k+1), Q22(:, i+1, k+1), c, -s);
                     end    
                 else  % {k = p}
                     [A(:, i, 1), A(:, i+1, 1)] = col_rot(A(:, i, 1), A(:, i+1, 1), c, -s);   
                     if ( flag_qmat == 1 )
                         [Q22(:, i, 1), Q22(:, i+1, 1)] = col_rot(Q22(:, i, 1), Q22(:, i+1, 1), c, -s);
                     end    
                 end    
                                
                 if ( k < p )
%                     [c, s] = givens(A(i, i, k+1), A(i+1, i, k+1));
                     [G, y] = planerot( [A(i, i, k+1); A(i+1, i, k+1)] );
                     c      = G(1,1);
                     s      = G(1,2);
                     [A(i, i: n3, k+1), A(i+1, i: n3, k+1)] = row_rot(A(i, i: n3, k+1), A(i+1, i: n3, k+1), c, s);
                     A(i+1, i, k+1) = 0;
                     [B(i, i: n3, k+1), B(i+1, i: n3, k+1)] = row_rot(B(i, i: n3, k+1), B(i+1, i: n3, k+1), c, s);
                     if ( flag_zmat == 1 )
                         [Z22(i, :, k+1), Z22(i+1, :, k+1)] = row_rot(Z22(i, :, k+1), Z22(i+1, :, k+1), c, s);
                     end
                 end %{if}
            end %{k}
        end %{i}
                                                     
        for i = n3: -1: n1        
 %           [c, s] = givens(A(i-1, 1, 1), A(i, 1, 1));
            [G, y] = planerot( [A(i-1, 1, 1); A(i, 1, 1)] );
            c      = G(1,1);
            s      = G(1,2);
            [A(i-1, :, 1), A(i, :, 1)] = row_rot(A(i-1, :, 1), A(i, :, 1), c, s);
            A(i, 1, 1) = 0;
            [B(i-1, i-1: n3, 1), B(i, i-1: n3, 1)] = row_rot(B(i-1, i-1: n3, 1), B(i, i-1: n3, 1), c, s);
            if ( flag_zmat == 1 )
                [Z22(i-1, :, 1), Z22(i, :, 1)] = row_rot(Z22(i-1, :, 1), Z22(i, :, 1), c, s);
            end
            
            for  k = 1: nb-1
%                 [c, s] = givens(B(i, i, k), B(i, i-1, k));
                 [G, y] = planerot( [B(i, i, k); B(i, i-1, k)] );
                 c      = G(1,1);
                 s      = G(1,2);
                 [B(1: i, i-1, k), B(1: i, i, k)] = col_rot(B(1: i, i-1, k), B(1: i, i, k), c, -s);
                 B(i, i-1, k) = 0;
                 [A(1: i, i-1, k+1), A(1: i, i, k+1)] = col_rot(A(1: i, i-1, k+1), A(1: i, i, k+1), c, -s);
                 if ( flag_qmat == 1 )
                     [Q22(:, i-1, k+1), Q22(:, i, k+1)] = col_rot(Q22(:, i-1, k+1), Q22(:, i, k+1), c, -s);
                 end    
                         
%                 [c, s] = givens(A(i-1, i-1, k+1), A(i, i-1, k+1));
                 [G, y] = planerot( [A(i-1, i-1, k+1); A(i, i-1, k+1)] );
                 c      = G(1,1);
                 s      = G(1,2);
                 [A(i-1, i-1: n3, k+1), A(i, i-1: n3, k+1)] = row_rot(A(i-1, i-1: n3, k+1), A(i, i-1: n3, k+1), c, s);
                 A(i, i-1, k+1) = 0;
                 if ( k < nb-1 )
                     [B(i-1, i-1: n3, k+1), B(i, i-1: n3, k+1)] = row_rot(B(i-1, i-1: n3, k+1), ... 
                      B(i, i-1: n3, k+1), c, s);
                 else    
                     [B(i-1, i: n3, nb), B(i, i: n3, nb)] = row_rot(B(i-1, i: n3, nb), ... 
                      B(i, i: n3, nb), c, s);
                 end    
                 if ( flag_zmat == 1 )
                     [Z22(i-1, :, k+1), Z22(i, :, k+1)] = row_rot(Z22(i-1, :, k+1), Z22(i, :, k+1), c, s);
                 end
            end %{k}
        end %{i}
                  
    end %{if}    
        
elseif ( flag_zero == 1 )
    
    if ( n1 == 1 )
        %disp('  Zero EW ( Case I ) ... ')
        for i = n11: n3
            for k = na: p  
                if ( k > na )
%                    [c, s] = givens(A(i-1, i-1, k), A(i, i-1, k));
                    [G, y] = planerot( [A(i-1, i-1, k); A(i, i-1, k)] );
                    c      = G(1,1);
                    s      = G(1,2);
                    [A(i-1, i-1: n3, k), A(i, i-1: n3, k)] = row_rot(A(i-1, i-1: n3, k), A(i, i-1: n3, k), c, s);
                    A(i, i-1, k) = 0;
                else
%                    [c, s] = givens(A(i-1, i, k), A(i, i, k));
                    [G, y] = planerot( [A(i-1, i, k); A(i, i, k)] );
                    c      = G(1,1);
                    s      = G(1,2);
                    [A(i-1, i: n3, k), A(i, i: n3, k)] = row_rot(A(i-1, i: n3, k), A(i, i: n3, k), c, s);
                    A(i, i, k) = 0;
                end    
                [B(i-1, i-1: n3, k), B(i, i-1: n3, k)] = row_rot(B(i-1, i-1: n3, k), B(i, i-1: n3, k), c, s);
                if ( flag_zmat == 1 )
                    [Z22(i-1, :, k), Z22(i, :, k)] = row_rot(Z22(i-1, :, k), Z22(i, :, k), c, s);
                end    
                 
%                [c, s] = givens(B(i, i, k), B(i, i-1, k));
                [G, y] = planerot( [B(i, i, k); B(i, i-1, k)] );
                c      = G(1,1);
                s      = G(1,2);
                [B(1: i, i-1, k), B(1: i, i, k)] = col_rot(B(1: i, i-1, k), B(1: i, i, k), c, -s);
                B(i, i-1, k) = 0;
                if ( k < p )
                    [A(1: i, i-1, k+1), A(1: i, i, k+1)] = col_rot(A(1: i, i-1, k+1), A(1: i, i, k+1), c, -s);
                    if ( flag_qmat == 1 )
                        [Q22(:, i-1, k+1), Q22(:, i, k+1)] = col_rot(Q22(:, i-1, k+1), Q22(:, i, k+1), c, -s);
                    end
                else
                    [A(:, i-1, 1), A(:, i, 1)] = col_rot(A(:, i-1, 1), A(:, i, 1), c, -s);
                    if ( flag_qmat == 1 )
                        [Q22(:, i-1, 1), Q22(:, i, 1)] = col_rot(Q22(:, i-1, 1), Q22(:, i, 1), c, -s);
                    end
                end    
                
            end %{k}
        end %{i}
        
        for j = n1: n11 
            for i = n3: -1: j+1        
%                [c, s] = givens(A(i-1, j, 1), A(i, j, 1));
                [G, y] = planerot( [A(i-1, j, 1); A(i, j, 1)] );
                c      = G(1,1);
                s      = G(1,2);
                [A(i-1, j: n3, 1), A(i, j: n3, 1)] = row_rot(A(i-1, j: n3, 1), A(i, j: n3, 1), c, s);
                A(i, j, 1) = 0;
                [B(i-1, i-1: n3, 1), B(i, i-1: n3, 1)] = row_rot(B(i-1, i-1: n3, 1), B(i, i-1: n3, 1), c, s);
                if ( flag_zmat == 1 )
                    [Z22(i-1, :, 1), Z22(i, :, 1)] = row_rot(Z22(i-1, :, 1), Z22(i, :, 1), c, s);
                end
                
                for  k = 1: na-1
%                     [c, s] = givens(B(i, i, k), B(i, i-1, k));
                     [G, y] = planerot( [B(i, i, k); B(i, i-1, k)] );
                     c      = G(1,1);
                     s      = G(1,2);
                     [B(1: i, i-1, k), B(1: i, i, k)] = col_rot(B(1: i, i-1, k), B(1: i, i, k), c, -s);
                     B(i, i-1, k) = 0;
                     if ( k < na-1 )
                         [A(1: i, i-1, k+1), A(1: i, i, k+1)] = col_rot(A(1: i, i-1, k+1), A(1: i, i, k+1), c, -s);
                     else
                         [A(1: n2, i-1, na), A(1: n2, i, na)] = col_rot(A(1: n2, i-1, na), A(1: n2, i, na), c, -s);    
                     end    
                     if ( flag_qmat == 1 )
                         [Q22(:, i-1, k+1), Q22(:, i, k+1)] = col_rot(Q22(:, i-1, k+1), Q22(:, i, k+1), c, -s);
                     end    
                     
                     if ( k < na-1 )
%                         [c, s] = givens(A(i-1, i-1, k+1), A(i, i-1, k+1));
                         [G, y] = planerot( [A(i-1, i-1, k+1); A(i, i-1, k+1)] );
                         c      = G(1,1);
                         s      = G(1,2);
                         [A(i-1, i-1: n3, k+1), A(i, i-1: n3, k+1)] = row_rot(A(i-1, i-1: n3, k+1), A(i, i-1: n3, k+1), c, s);
                         A(i, i-1, k+1) = 0;
                         [B(i-1, i-1: n3, k+1), B(i, i-1: n3, k+1)] = row_rot(B(i-1, i-1: n3, k+1), B(i, i-1: n3, k+1), c, s);
                         if ( flag_zmat == 1 )
                             [Z22(i-1, :, k+1), Z22(i, :, k+1)] = row_rot(Z22(i-1, :, k+1), Z22(i, :, k+1), c, s);
                         end
                     end %{if}    
                end %{k}
            end %{i}
        end %{j}
        
        for k = na: p  
%            [c, s] = givens(A(n1, n1, k), A(n11, n1, k));
            [G, y] = planerot( [A(n1, n1, k); A(n11, n1, k)] );
            c      = G(1,1);
            s      = G(1,2);
            [A(n1, :, k), A(n11, :, k)] = row_rot(A(n1, :, k), A(n11, :, k), c, s);
            A(n11, n1, k) = 0;
            [B(n1, :, k), B(n11, :, k)] = row_rot(B(n1, :, k), B(n11, :, k), c, s);
            if ( flag_zmat == 1 )
                [Z22(n1, :, k), Z22(n11, :, k)] = row_rot(Z22(n1, :, k), Z22(n11, :, k), c, s);
            end    
                 
%            [c, s] = givens(B(n11, n11, k), B(n11, n1, k));
            [G, y] = planerot( [B(n11, n11, k); B(n11, n1, k)] );
            c      = G(1,1);
            s      = G(1,2);
            [B(1: n11, n1, k), B(1: n11, n11, k)] = col_rot(B(1: n11, n1, k), B(1: n11, n11, k), c, -s);
            B(n11, n1, k) = 0;
            if ( k < p )
                [A(1: n11, n1, k+1), A(1: n11, n11, k+1)] = col_rot(A(1: n11, n1, k+1), A(1: n11, n11, k+1), c, -s);
                if ( flag_qmat == 1 )
                    [Q22(:, n1, k+1), Q22(:, n11, k+1)] = col_rot(Q22(:, n1, k+1), Q22(:, n11, k+1), c, -s);
                end
            else
                [A(1: n11, n1, 1), A(1: n11, n11, 1)] = col_rot(A(1: n11, n1, 1), A(1: n11, n11, 1), c, -s);
                if ( flag_qmat == 1 )
                    [Q22(:, n1, 1), Q22(:, n11, 1)] = col_rot(Q22(:, n1, 1), Q22(:, n11, 1), c, -s);
                end
            end    
        end %{k}
            
    elseif ( n2 == 1 )
        
        %disp('  Zero EW ( Case II ) ... ')
        for i = n1: -1: 1
            for k = na: -1: 2  
                if ( k < na )
%                    [c, s] = givens(A(i+1, i+1, k), A(i+1, i, k));
                    [G, y] = planerot( [A(i+1, i+1, k); A(i+1, i, k)] );
                    c      = G(1,1);
                    s      = G(1,2);
                    [A(1: i+1, i, k), A(1: i+1, i+1, k)] = col_rot(A(1: i+1, i, k), A(1: i+1, i+1, k), c, -s);
                    A(i+1, i, k) = 0;
                else
%                    [c, s] = givens(A(i, i+1, k), A(i, i, k));
                    [G, y] = planerot( [A(i, i+1, k); A(i, i, k)] );
                    c      = G(1,1);
                    s      = G(1,2);
                    [A(1: i, i, k), A(1: i, i+1, k)] = col_rot(A(1: i, i, k), A(1: i, i+1, k), c, -s);
                    A(i, i, k) = 0;
                end    
                [B(1: i+1, i, k-1), B(1: i+1, i+1, k-1)] = col_rot(B(1: i+1, i, k-1), B(1: i+1, i+1, k-1), c, -s);
                if ( flag_qmat == 1 )
                    [Q22(:, i, k), Q22(:, i+1, k)] = col_rot(Q22(:, i, k), Q22(:, i+1, k), c, -s);
                end    
                 
                [c, s] = givens(B(i, i, k-1), B(i+1, i, k-1));
                [B(i, i: n3, k-1), B(i+1, i: n3, k-1)] = row_rot(B(i, i: n3, k-1), B(i+1, i: n3, k-1), c, s);
                B(i+1, i, k-1) = 0;
                if ( k > 2 )
                    [A(i, i: n3, k-1), A(i+1, i: n3, k-1)] = row_rot(A(i, i: n3, k-1), A(i+1, i: n3, k-1), c, s);
                else
                    [A(i, :, 1), A(i+1, :, 1)] = row_rot(A(i, :, 1), A(i+1, :, 1), c, s);
                end
                if ( flag_zmat == 1 )
                    [Z22(i, :, k-1), Z22(i+1, :, k-1)] = row_rot(Z22(i, :, k-1), Z22(i+1, :, k-1), c, s);
                end
                
            end %{k}
        end %{i}
        
        for i = n3: -1: n1 
            for j = 1: i-1        
                [c, s] = givens(A(i, j+1, 1), A(i, j, 1));
                [A(1: i, j, 1), A(1: i, j+1, 1)] = col_rot(A(1: i, j, 1), A(1: i, j+1, 1), c, -s);
                A(i, j, 1) = 0;
                [B(1: j+1, j, p), B(1: j+1, j+1, p)] = col_rot(B(1: j+1, j, p), B(1: j+1, j+1, p), c, -s);
                if ( flag_qmat == 1 )
                    [Q22(:, j, 1), Q22(:, j+1, 1)] = col_rot(Q22(:, j, 1), Q22(:, j+1, 1), c, -s);
                end
                
                for  k = p: -1: na
                     [c, s] = givens(B(j, j, k), B(j+1, j, k));
                     [B(j, j: n3, k), B(j+1, j: n3, k)] = row_rot(B(j, j: n3, k), B(j+1, j: n3, k), c, s);
                     B(j+1, j, k) = 0;
                     if ( k > na )
                         [A(j, j: n3, k), A(j+1, j: n3, k)] = row_rot(A(j, j: n3, k), A(j+1, j: n3, k), c, s);
                     else
                         [A(j, n1: n3, k), A(j+1, n1: n3, k)] = row_rot(A(j, n1: n3, k), A(j+1, n1: n3, k), c, s);   
                     end    
                     if ( flag_zmat == 1 )
                         [Z22(j, :, k), Z22(j+1, :, k)] = row_rot(Z22(j, :, k), Z22(j+1, :, k), c, s);
                     end    
                     
                     if ( k > na )
                         [c, s] = givens(A(j+1, j+1, k), A(j+1, j, k));
                         [A(1: j+1, j, k), A(1: j+1, j+1, k)] = col_rot(A(1: j+1, j, k), A(1: j+1, j+1, k), c, -s);
                         A(j+1, j, k) = 0;
                         [B(1: j+1, j, k-1), B(1: j+1, j+1, k-1)] = col_rot(B(1: j+1, j, k-1), B(1: j+1, j+1, k-1), c, -s);
                         if ( flag_qmat == 1 )
                             [Q22(:, j, k), Q22(:, j+1, k)] = col_rot(Q22(:, j, k), Q22(:, j+1, k), c, -s);
                         end
                     end %{if}    
                end %{k}
            end %{j}
        end %{i}
        
        for k = na: -1: 2  
            [c, s] = givens(A(n3, n3, k), A(n3, n1, k));
            [A(:, n1, k), A(:, n3, k)] = col_rot(A(:, n1, k), A(:, n3, k), c, -s);
            A(n3, n1, k) = 0;
            [B(:, n1, k-1), B(:, n3, k-1)] = col_rot(B(:, n1, k-1), B(:, n3, k-1), c, -s);
            if ( flag_qmat == 1 )
                [Q22(:, n1, k), Q22(:, n3, k)] = col_rot(Q22(:, n1, k), Q22(:, n3, k), c, -s);
            end    
                 
            [c, s] = givens(B(n1, n1, k-1), B(n3, n1, k-1));
            [B(n1, n1: n3, k-1), B(n3, n1: n3, k-1)] = row_rot(B(n1, n1: n3, k-1), B(n3, n1: n3, k-1), c, s);
            B(n3, n1, k-1) = 0;
            [A(n1, n1: n3, k-1), A(n3, n1: n3, k-1)] = row_rot(A(n1, n1: n3, k-1), A(n3, n1: n3, k-1), c, s);
            if ( flag_zmat == 1 )
                [Z22(n1, :, k-1), Z22(n3, :, k-1)] = row_rot(Z22(n1, :, k-1), Z22(n3, :, k-1), c, s);
            end
        end %{k}
    
    end %{if}    
        
else % No zero or inf Ews.
    
    %disp('  Normal case: NO zero or infinity ew ... ')
    S = B(:, :, p) \ A(:, :, p) ;
    for k = (p-1): -1: 1
        BA = B(:, :, k) \ A(:, :, k) ;
        S = S * BA ;    
    end    
        
    S11 = S(1: n1, 1: n1) ;
    S12 = S(1: n1, n11: n3) ;
    S22 = S(n11: n3, n11: n3) ;
    r = S12(:) ; 
    
    K = kron(In2, S11) - kron(S22', In1); 
    [L,H,P,Q] = lu_complpiv(K);
    e = min(abs(diag(H)));
    sig_n1 = 1: n1*n2;
    for k = 1: n1*n2-1;
        sig_n1([k,P(k)]) = sig_n1([P(k),k]);
    end 
    r = e * r(sig_n1); 
    x = (H\(L\r));
    sig_n2 = 1: n1*n2;
    for k = 1: n1*n2-1;
        sig_n2([k,Q(k)]) = sig_n2([Q(k),k]);  
    end
    x(sig_n2) = x;
    X = [];
    for j = 1: n2
        X = [X, x((j-1)*n1+1:j*n1)];
    end
    M = [-X; e*In2];  
    X = X ./ e;
    
    % Compute residual and backward error of the solution X to a Sylvester eq.
%     RR = S12 - S11 * X + X * S22; 
%     norm_RR = norm(RR,'fro');
%     aa = norm(S11,'fro'); bb = norm(S22,'fro'); rr = norm(S12,'fro');
%     xx = norm(X,'fro');
%     dn_X = ( aa + bb ) * xx + rr ;
%     res_X = norm_RR / dn_X ; 
%     msv = min(svd(X));
%     aa2 = aa * msv;  bb2 = bb * msv;
%     dn_mu = sqrt(aa2*aa2 + bb2*bb2 + rr*rr );
%     bakerr_X = norm_RR / dn_mu;
%     res_bakerr_X = [res_X bakerr_X]
            
    for j = 1: n2
                
        for i = (n3-1): -1: j 
            
            [c, s] = givens(M(i,j), M(i+1,j));
            if ( s ~= 0 )
                [M(i, j: n2), M(i+1, j: n2)] = row_rot(M(i, j: n2), M(i+1, j: n2), c, s) ;
                M(i+1, j) = 0;
                [A(1: n3, i, 1), A(1: n3, i+1, 1)] = col_rot(A(1: n3, i, 1), A(1: n3, i+1, 1), c, s);
                if ( flag_qmat == 1 )
                    [Q22(:, i, 1), Q22(:, i+1, 1)] = col_rot(Q22(:, i, 1), Q22(:, i+1, 1), c, s);    
                end    
                [B(1: i+1, i, p), B(1: i+1, i+1, p)] = col_rot(B(1: i+1, i, p), B(1: i+1, i+1, p), c, s);
                
                for k = p: -1: 1
                    [c, s] = givens(B(i, i, k), B(i+1, i, k)) ;
                    [B(i, i: n3, k), B(i+1, i: n3, k)] = row_rot(B(i, i: n3, k), B(i+1, i: n3, k), c, s) ;
                    if ( flag_zmat == 1 )
                        [Z22(i, :, k), Z22(i+1, :, k)] = row_rot(Z22(i, :, k), Z22(i+1, :, k), c, s) ;    
                    end    
                    B(i+1, i, k) = 0;
                    if ( k > 1 )
                        [A(i, i: n3, k), A(i+1, i: n3, k)] = row_rot(A(i, i: n3, k), A(i+1, i: n3, k), c, s) ;
                    else
                        [A(i, 1: n3, k), A(i+1, 1: n3, k)] = row_rot(A(i, 1: n3, k), A(i+1, 1: n3, k), c, s) ;
                    end    
                    
                    if ( k > 1 )
                        [c, s] = givens(A(i+1, i+1, k), A(i+1, i, k)) ;
                        [A(1: i+1, i, k), A(1: i+1, i+1, k)] = col_rot(A(1: i+1, i, k), A(1: i+1, i+1, k), c, -s) ;
                        if ( flag_qmat == 1 )
                            [Q22(:, i, k), Q22(:, i+1, k)] = col_rot(Q22(:, i, k), Q22(:, i+1, k), c, -s) ;    
                        end    
                        A(i+1, i, k) = 0;
                        [B(1: i+1, i, k-1), B(1: i+1, i+1, k-1)] = col_rot(B(1: i+1, i, k-1), B(1: i+1, i+1, k-1), c, -s) ;
                    end %{if}    
                end %{k}    
            end %{if}    
                        
        end %{i}
        
    end %{j} 
    
%     disp('  Before the post-procedure of reordering ews, ... ')
%     A1 = A(:,:,1)
%     B
%     disp('***********************************************************************')
%     pause
    
    %  Reproduce the periodic Schur form (PSF) of matrix pair {A_k, B_k}, k = 1: p.
    if ( n1 == 1 ) & ( n2 == 1 )  % Two real ews.
        
        tmp_b = 1;  bdp = 1;  
        Emat = eye(2); ew22 = zeros(2, 1);
        for k = p: -1: 1
            tmp_b = B(1, 1, k) * B(2, 2, k);
            bdp = bdp * tmp_b;
            tmp_B = [B(2, 2, k)  -B(1, 2, k);
                          0       B(1, 1, k)];
            tmp_B = tmp_B * A(:, :, k);
            Emat = Emat * tmp_B;
        end %{k}   
        Emat = Emat ./ bdp ;
        %ew22 = eig(Emat) ;
        b = Emat(1, 1) + Emat(2, 2); c = det(Emat);  
        if ( sign(b) == 0 )
            ew22 = [sqrt(-c); -sqrt(-c)];
        else
            tmp = sqrt(b*b-4*c);
            dn = b + sign(b) * tmp;
            ew22(1, 1) = dn / 2;
            ew22(2, 1) = (2*c) / dn;
        end    
        
        maxite = 5;
        
        for ite = 1: maxite
            
            a11 = abs( A(1, 1, 1) );
            a22 = abs( A(2, 2, 1) );
            a21 = abs( A(2, 1, 1) ); 
            err = tol * ( a11 + a22 );
            if ( a21 <= err )
                A(2, 1, 1) = 0;
                break;
            end   
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %    
            %  Evaluate a single shift --- lam.
            %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            lam = A(2, 2, 1) * B(1, 1, 1);
            b_11 = B(1, 1, 1);
            a_11 = 1;
            for k = 2: p
                lam = lam * A(2, 2, k) * B(1, 1, k);
                b_11 = b_11 * B(1, 1, k);
                a_11 = a_11 * A(1, 1, k);
            end
            lam = lam / bdp ;
            [ss, I22] = min(abs(ew22 - lam)) ;
            lam = ew22(I22, 1) ;
            w2 = A(:, 1, 1) ;
            w2(1, 1) = w2(1, 1) - (lam * b_11) / a_11 ;
            w2 = real(w2) ;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %
            %  Periodic QR step with single shift 
            %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            [vk2, bk] = house(w2);
            if ( bk ~= 0 )
                bv2 = bk * vk2;
                A(:, :, 1) = A(:, :, 1) - bv2 * ( vk2' * A(:, :, 1) );
                if ( flag_zmat == 1 )
                    Z22(:, :, 1) = Z22(:, :, 1) - bv2 * ( vk2' * Z22(:, :, 1) );  
                end 
                B(:, :, 1) = B(:, :, 1) - bv2 * ( vk2' * B(:, :, 1) );
            end
            
            for k = 1: p
                       
                w2 = [B(2, 2, k); B(2, 1, k)];
                [vk2, bk] = house(w2);
                if ( bk ~= 0 )
                    vk2 = flipud(vk2);
                    bv2 = bk * vk2;
                    B(:, :, k) = B(:, :, k) - ( B(:, :, k) * vk2 ) * bv2';
                    B(2, 1, k) = 0;
                    if ( k < p )
                        A(:, :, k+1) = A(:, :, k+1) - ( A(:, :, k+1) * vk2 ) * bv2';
                        if ( flag_qmat == 1 )
                            Q22(:, :, k+1) = Q22(:, :, k+1) - ( Q22(:, :, k+1) * vk2 ) * bv2';  
                        end
                    else %{k = p}
                        A(:, :, 1) = A(:, :, 1) - ( A(:, :, 1) * vk2 ) * bv2';
                        if ( flag_qmat == 1 )
                            Q22(:, :, 1) = Q22(:, :, 1) - ( Q22(:, :, 1) * vk2 ) * bv2';  
                        end
                    end %{if}    
                end  %{if}      
                
                if ( k < p )
                    w2 = [A(1, 1, k+1); A(2, 1, k+1)];
                    [vk2, bk] = house(w2);
                    if ( bk ~= 0 )
                        bv2 = bk * vk2;
                        A(:, :, k+1) = A(:, :, k+1) - bv2 * ( vk2' * A(:, :, k+1) );
                        A(2, 1, k+1) = 0;
                        if ( flag_zmat == 1 )
                            Z22(:, :, k+1) = Z22(:, :, k+1) - bv2 * ( vk2' * Z22(:, :, k+1) );  
                        end 
                        B(:, :, k+1) = B(:, :, k+1) - bv2 * ( vk2' * B(:, :, k+1) );
                    end %{if}
                end %{if}  
                           
            end % {for k}
               
        end %{ite}    
        
    else % Other three cases.    
        
        do_pqr = 0;
        if ( n1 == 1 ) & ( n2 == 2 )
            a21 = abs(A(3, 2, 1)) ;
            a11 = abs(A(2, 2, 1)) ;
            a22 = abs(A(3, 3, 1)) ;
            err = tol * (a11 + a22) ;
            if ( a21 <= err )
                A(3, 2, 1) = 0 ; A(3, 1, 1) = 0;
            else    
                do_pqr = 1 ;
            end % {if}
        elseif ( n1 == 2 )
            if ( n2 == 1 )
                a21 = abs(A(2, 1, 1)) ;
                a11 = abs(A(1, 1, 1)) ;
                a22 = abs(A(2, 2, 1)) ;
                err = tol * (a11 + a22) ;
                if ( a21 <= err )
                    A(2, 1, 1) = 0 ; A(3, 1, 1) = 0;
                else    
                    do_pqr = 1 ;
                end % {if}
            elseif ( n2 == 2 )
                a21 = abs(A(3, 2, 1)) ;
                a11 = abs(A(2, 2, 1)) ;
                a22 = abs(A(3, 3, 1)) ;
                err = tol * (a11 + a22) ;
                if ( a21 <= err )
                    A(3:4, 1:2, 1) = zeros(2); 
                else    
                    do_pqr = 1 ;
                end % {if}                
            end %{if}    
        end  %{if}   
                        
        if ( do_pqr == 1 )
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %
            %  Transform the swapped periodic matrix pair {Ai, Bi}, i = 1: p, into new matrix product 
            %      
            %                Bp^{-1} * Ap * ... * B1^{-1} * A1,
            %           
            %  where matrices Ai and Bj are upper triangular for i = 2: p, j = 1: p.
            %
            %  Moreover, A1 is upper Hessenberg.
            %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            n31 = n3 - 1;  n32 = n3 - 2;
            for j = 1: n32
                for i = n3: -1: (j+2) 
                    [c, s] = givens(A(i-1, j, 1), A(i, j, 1));
                    [A(i-1, j: n3, 1), A(i, j: n3, 1)] = row_rot( A(i-1, j: n3, 1), A(i, j: n3, 1), c, s );
                    A(i, j, 1) = 0;
                    [B(i-1, i-1: n3, 1), B(i, i-1: n3, 1)] = row_rot( B(i-1, i-1: n3, 1), B(i, i-1: n3, 1), c, s );
                    if ( flag_zmat == 1 )
                        [Z22(i-1, :, 1), Z22(i, :, 1)] = row_rot( Z22(i-1, :, 1), Z22(i, :, 1), c, s ); 
                    end %{if}    
                    
                    [c, s] = givens( B(i, i, 1), B(i, i-1, 1) );
                    [B(1:i, i-1, 1), B(1:i, i, 1)] = col_rot( B(1:i, i-1, 1), B(1:i, i, 1), c, -s );
                    B(i, i-1, 1) = 0 ;
                    [A(1:i, i-1, 2), A(1:i, i, 2)] = col_rot( A(1:i, i-1, 2), A(1:i, i, 2), c, -s );   
                    if ( flag_qmat == 1 )
                        [Q22(:, i-1, 2), Q22(:, i, 2)] = col_rot( Q22(:, i-1, 2), Q22(:, i, 2), c, -s );    
                    end
                                    
                    for k = 2: p
                        [c, s] = givens(A(i-1, i-1, k), A(i, i-1, k));
                        [A(i-1, i-1: n3, k), A(i, i-1: n3, k)] = row_rot( A(i-1, i-1: n3, k), A(i, i-1: n3, k), c, s );
                        A(i, i-1, k) = 0;
                        [B(i-1, i-1: n3, k), B(i, i-1: n3, k)] = row_rot( B(i-1, i-1: n3, k), B(i, i-1: n3, k), c, s );
                        if ( flag_zmat == 1 )
                            [Z22(i-1, :, k), Z22(i, :, k)] = row_rot( Z22(i-1, :, k), Z22(i, :, k), c, s ); 
                        end %{if}    
                        
                        [c, s] = givens( B(i, i, k), B(i, i-1, k) );
                        [B(1:i, i-1, k), B(1:i, i, k)] = col_rot( B(1:i, i-1, k), B(1:i, i, k), c, -s );
                        B(i, i-1, k) = 0 ;
                        if ( k < p )
                            [A(1:i, i-1, k+1), A(1:i, i, k+1)] = col_rot( A(1:i, i-1, k+1), A(1:i, i, k+1), c, -s );   
                            if ( flag_qmat == 1 )
                                [Q22(:, i-1, k+1), Q22(:, i, k+1)] = col_rot( Q22(:, i-1, k+1), Q22(:, i, k+1), c, -s );    
                            end
                        else % { k = p }
                            [A(:, i-1, 1), A(:, i, 1)] = col_rot( A(:, i-1, 1), A(:, i, 1), c, -s );   
                            if ( flag_qmat == 1 )
                                [Q22(:, i-1, 1), Q22(:, i, 1)] = col_rot( Q22(:, i-1, 1), Q22(:, i, 1), c, -s );    
                            end
                        end  % {if}
                    end %{k}    
                end %{i}        
            end %{j}    
            
            flag_pqr = 0;
            while ( flag_pqr == 0 )
               
               for i = 2: n3
                   a21 = abs(A(i, i-1, 1)) ;
                   a11 = abs(A(i-1, i-1, 1)) ;
                   a22 = abs(A(i, i, 1)) ;
                   err = tol * (a11 + a22) ;
                   if ( a21 <= err )
                       A(i, i-1, 1) = 0 ;
                       flag_pqr = 1 ;
                   end % {if}
               end % {for}
                                                    
               if ( flag_pqr == 0 )
                   
                   %  Do PQR Step
                   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                   %
                   %  Calculation of double shifts
                   %
                   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                   
                   Emat = eye(2);  bdp = 1;  tmp_B = zeros(2);
                   for k = p: -1: 1
                       tmp_b = B(n31, n31, k) * B(n3, n3, k);
                       bdp = bdp * tmp_b;
                       tmp_B = [B(n3, n3, k)  -B(n31, n3, k);
                                    0         B(n31, n31, k)];
                       tmp_B = tmp_B * A(n31: n3, n31: n3, k);
                       Emat = Emat * tmp_B;
                   end %{k}   
                   Emat = Emat ./ bdp;
                   ss = Emat(1, 1) + Emat(2, 2) ;
                   sp = det(Emat); 
                   
                   bdp = B(1, 1, 1) * B(2, 2, 1);
                   vh = [B(2, 2, 1)  -B(1, 2, 1);
                              0       B(1, 1, 1)] * A(1: 2, 1, 1);
                   for k = 2: p
                       tmp_b = B(1, 1, k) * B(2, 2, k);
                       bdp = bdp * tmp_b;
                       vh = A(1:2, 1:2, k) * vh ;
                       vh = [B(2, 2, k)  -B(1, 2, k);
                                  0       B(1, 1, k)] * vh ;
                   end %{k}
                   vh = vh ./ bdp ;
                   
                   w1 = A(1:3, 1:2, 1) * vh;  w1 = B(1:3, 1:3, 1) \ w1;
                   for k = 2: p
                       w1 = A(1:3, 1:3, k) * w1 ;
                       w1 = B(1:3, 1:3, k) \ w1 ;
                   end %{k}
                   w1(1, 1) = w1(1, 1) + sp - ( ss * vh(1, 1) ) ;
                   w1(2, 1) = w1(2, 1) - ( ss * vh(2, 1) ) ; 
                                      
                   %%%%%%%%%%%%%%%%%%%%%
                   %    
                   %  Periodic QR step 
                   %    
                   %%%%%%%%%%%%%%%%%%%%%
                   
                   [vk1, bk] = house(w1);
                   if ( bk ~= 0 )
                       bv1 = bk * vk1;
                       B(1:3, 1:3, p) = B(1:3, 1:3, p) - ( B(1:3, 1:3, p) * vk1 ) * bv1' ;
                       A(:, 1:3, 1) = A(:, 1:3, 1) - ( A(:, 1:3, 1) * vk1 ) * bv1' ;
                       if ( flag_qmat == 1 )
                           Q22(:, 1:3, 1) = Q22(:, 1:3, 1) - ( Q22(:, 1:3, 1) * vk1 ) * bv1';  
                       end
                   end
                   
                   for k = p: -1: 1
                       w1 = B(1:3, 1, k); 
                       [vk1, bk] = house(w1);
                       if ( bk ~= 0 )
                           bv1 = bk * vk1; 
                           B(1:3, 1:n3, k) = B(1:3, 1:n3, k) - bv1 * ( vk1' * B(1:3, 1:n3, k)  ) ;
                           B(2, 1, k) = 0; B(3, 1, k) = 0;
                           A(1:3, 1:n3, k) = A(1:3, 1:n3, k) - bv1 * ( vk1' * A(1:3, 1:n3, k)  ) ;
                           if ( flag_zmat == 1 )
                               Z22(1:3, :, k) = Z22(1:3, :, k) - bv1 * ( vk1' * Z22(1:3, :, k)  ) ;
                           end
                       end    
                       
                       w2 = B(2:3, 2, k); 
                       [vk2, bk] = house(w2);
                       if ( bk ~= 0 )
                           bv2 = bk * vk2; 
                           B(2:3, 2:n3, k) = B(2:3, 2:n3, k) - bv2 * ( vk2' * B(2:3, 2:n3, k)  ) ;
                           B(3, 2, k) = 0;
                           A(2:3, 1:n3, k) = A(2:3, 1:n3, k) - bv2 * ( vk2' * A(2:3, 1:n3, k)  ) ;
                           if ( flag_zmat == 1 )
                               Z22(2:3, :, k) = Z22(2:3, :, k) - bv2 * ( vk2' * Z22(2:3, :, k)  ) ;
                           end
                       end
                                  
                       if ( k > 1 )
                           w1 = [A(3, 3, k); A(3, 2, k); A(3, 1, k)];
                           [vk1, bk] = house(w1);
                           if ( bk ~= 0 )
                               vk1 = flipud(vk1);
                               bv1 = bk * vk1; 
                               A(1:3, 1:3, k) = A(1:3, 1:3, k) - ( A(1:3, 1:3, k) * vk1 ) * bv1';
                               A(3, 2, p) = 0;  A(3, 1, p) = 0;
                               B(1:3, 1:3, k-1) = B(1:3, 1:3, k-1) - ( B(1:3, 1:3, k-1) * vk1 ) * bv1';
                               if ( flag_qmat == 1 )
                                   Q22(:, 1:3, k) = Q22(:, 1:3, k) - ( Q22(:, 1:3, k) * vk1 ) * bv1';  
                               end
                           end    
                                                                  
                           w2 = [A(2, 2, k); A(2, 1, k)];
                           [vk2, bk] = house(w2);
                           if ( bk ~= 0 )
                               vk2 = flipud(vk2);
                               bv2 = bk * vk2; 
                               A(1:2, 1:2, k) = A(1:2, 1:2, k) - ( A(1:2, 1:2, k) * vk2 ) * bv2';
                               A(2, 1, k) = 0 ;
                               B(1:3, 1:2, k-1) = B(1:3, 1:2, k-1) - ( B(1:3, 1:2, k-1) * vk2 ) * bv2';
                               if ( flag_qmat == 1 )
                                   Q22(:, 1:2, k) = Q22(:, 1:2, k) - ( Q22(:, 1:2, k) * vk2 ) * bv2';  
                               end
                           end
                       end %{if}    
                   end %{for k}    
                                          
                   for i = 2: n32
                       w1 = [A(i, i-1, 1); A(i+1, i-1, 1); A(i+2, i-1, 1)];
                       [vk1, bk] = house(w1);
                       if ( bk ~= 0 )
                           bv1 = bk * vk1;
                           A(i: i+2, i-1: n3, 1) = A(i: i+2, i-1: n3, 1) - bv1 * ( vk1' * A(i: i+2, i-1: n3, 1) );
                           A(i+1, i-1, 1) = 0;  A(i+2, i-1, 1) = 0;
                           if ( flag_zmat == 1 )
                               Z22(i: i+2, :, 1) = Z22(i: i+2, :, 1) - bv1 * ( vk1' * Z22(i: i+2, :, 1) );  
                           end 
                           B(i: i+2, i: n3, 1) = B(i: i+2, i: n3, 1) - bv1 * ( vk1' * B(i: i+2, i: n3, 1) );
                       end
                       
                       for k = 1: p
                           w1 = [B(i+2, i+2, k); B(i+2, i+1, k); B(i+2, i, k)];
                           [vk1, bk] = house(w1);
                           if ( bk ~= 0 )
                               vk1 = flipud(vk1);
                               bv1 = bk * vk1; 
                               B(1: i+2, i: i+2, k) = B(1: i+2, i: i+2, k) - ( B(1: i+2, i: i+2, k) * vk1 ) * bv1';
                               B(i+2, i+1, k) = 0;  B(i+2, i, k) = 0;
                               if ( k < p )
                                   A(1: i+2, i: i+2, k+1) = A(1: i+2, i: i+2, k+1) - ( A(1: i+2, i: i+2, k+1) * vk1 ) * bv1';   
                                   if ( flag_qmat == 1 )
                                       Q22(:, i: i+2, k+1) = Q22(:, i: i+2, k+1) - ( Q22(:, i: i+2, k+1) * vk1 ) * bv1';  
                                   end
                               else %{k = p}
                                   A(:, n32: n3, 1) = A(:, n32: n3, 1) - ( A(:, n32: n3, 1) * vk1 ) * bv1';
                                   if ( flag_qmat == 1 )
                                       Q22(:, n32: n3, 1) = Q22(:, n32: n3, 1) - ( Q22(:, n32: n3, 1) * vk1 ) * bv1';  
                                   end
                               end %{if}   
                           end %{if}        
                             
                           w2 = [B(i+1, i+1, k); B(i+1, i, k)];
                           [vk2, bk] = house(w2);
                           if ( bk ~= 0 )
                               vk2 = flipud(vk2);
                               bv2 = bk * vk2;
                               B(1: i+1, i: i+1, k) = B(1: i+1, i: i+1, k) - ( B(1: i+1, i: i+1, k) * vk2 ) * bv2';
                               B(i+1, i, k) = 0;
                               if ( k < p )
                                   A(1: i+2, i: i+1, k+1) = A(1: i+2, i: i+1, k+1) - ( A(1: i+2, i: i+1, k+1) * vk2 ) * bv2';
                                   if ( flag_qmat == 1 )
                                       Q22(:, i: i+1, k+1) = Q22(:, i: i+1, k+1) - ( Q22(:, i: i+1, k+1) * vk2 ) * bv2';  
                                   end        
                               else %{k = p}
                                   A(:, n32: n31, 1) = A(:, n32: n31, 1) - ( A(:, n32: n31, 1) * vk2 ) * bv2';
                                   if ( flag_qmat == 1 )
                                       Q22(:, n32: n31, 1) = Q22(:, n32: n31, 1) - ( Q22(:, n32: n31, 1) * vk2 ) * bv2';  
                                   end    
                               end %{if}    
                           end  %{if}      
                            
                           if ( k < p )
                               w1 = [A(i, i, k+1); A(i+1, i, k+1); A(i+2, i, k+1)];
                               [vk1, bk] = house(w1);
                               if ( bk ~= 0 )
                                   bv1 = bk * vk1;
                                   A(i: i+2, i: n3, k+1) = A(i: i+2, i: n3, k+1) - bv1 * ( vk1' * A(i: i+2, i: n3, k+1) );
                                   A(i+1, i, k+1) = 0;  A(i+2, i, k+1) = 0;
                                   if ( flag_zmat == 1 )
                                       Z22(i: i+2, :, k+1) = Z22(i: i+2, :, k+1) - bv1 * ( vk1' * Z22(i: i+2, :, k+1) );  
                                   end 
                                   B(i: i+2, i: n3, k+1) = B(i: i+2, i: n3, k+1) - bv1 * ( vk1' * B(i: i+2, i: n3, k+1) );
                               end
                               
                               w2 = [A(i+1, i+1, k+1); A(i+2, i+1, k+1)];
                               [vk2, bk] = house(w2);
                               if ( bk ~= 0 )
                                   bv2 = bk * vk2;
                                   A(i+1: i+2, i+1: n3, k+1) = A(i+1: i+2, i+1: n3, k+1) - bv2 * ( vk2' * A(i+1: i+2, i+1: n3, k+1) );
                                   A(i+2, i+1, k+1) = 0;
                                   if ( flag_zmat == 1 )
                                       Z22(i+1: i+2, :, k+1) = Z22(i+1: i+2, :, k+1) - bv2 * ( vk2' * Z22(i+1: i+2, :, k+1) );  
                                   end 
                                   B(i+1: i+2, i: n3, k+1) = B(i+1: i+2, i: n3, k+1) - bv2 * ( vk2' * B(i+1: i+2, i: n3, k+1) );
                               end 
                           end %{if}  
                       end % {for k}
                   end % {for i}
                   
                   %
                   %  i = n3 - 1
                   %
                   w2 = [A(n31, n32, 1); A(n3, n32, 1)]; 
                   [vk2, bk] = house(w2);
                   if ( bk ~= 0 )
                       bv2 = bk * vk2;
                       A(n31: n3, n32: n3, 1) = A(n31: n3, n32: n3, 1) - bv2 * ( vk2' * A(n31: n3, n32: n3, 1) );
                       A(n3, n32, 1) = 0;
                       B(n31: n3, n31: n3, 1) = B(n31: n3, n31: n3, 1) - bv2 * ( vk2' * B(n31: n3, n31: n3, 1) );
                       if ( flag_zmat == 1 )
                           Z22(n31: n3, :, 1) = Z22(n31: n3, :, 1) - bv2 * ( vk2' * Z22(n31: n3, :, 1) );       
                       end 
                   end %{if}
                   
                   for k = 1: p
                       w2 = [B(n3, n3, k); B(n3, n31, k)];
                       [vk2, bk] = house(w2);
                       if ( bk ~= 0 )
                           vk2 = flipud(vk2);
                           bv2 = bk * vk2;
                           B(:, n31: n3, k) = B(:, n31: n3, k) - ( B(:, n31: n3, k) * vk2 ) * bv2';
                           B(n3, n31, k) = 0;
                           if ( k < p )
                               A(:, n31: n3, k+1) = A(:, n31: n3, k+1) - ( A(:, n31: n3, k+1) * vk2 ) * bv2';
                               if ( flag_qmat == 1 )
                                   Q22(:, n31: n3, k+1) = Q22(:, n31: n3, k+1) - ( Q22(:, n31: n3, k+1) * vk2 ) * bv2';  
                               end
                           else %{k = p}
                               A(:, n31: n3, 1) = A(:, n31: n3, 1) - ( A(:, n31: n3, 1) * vk2 ) * bv2';
                               if ( flag_qmat == 1 )
                                   Q22(:, n31: n3, 1) = Q22(:, n31: n3, 1) - ( Q22(:, n31: n3, 1) * vk2 ) * bv2';  
                               end
                           end %{if}    
                       end  %{if}      
                       
                       if ( k < p )
                           w2 = [A(n31, n31, k+1); A(n3, n31, k+1)];
                           [vk2, bk] = house(w2);
                           if ( bk ~= 0 )
                               bv2 = bk * vk2;
                               A(n31: n3, n31: n3, k+1) = A(n31: n3, n31: n3, k+1) - bv2 * ( vk2' * A(n31: n3, n31: n3, k+1) );
                               A(n3, n31, k+1) = 0;
                               if ( flag_zmat == 1 )
                                   Z22(n31: n3, :, k+1) = Z22(n31: n3, :, k+1) - bv2 * ( vk2' * Z22(n31: n3, :, k+1) );  
                               end
                               B(n31: n3, n31: n3, k+1) = B(n31: n3, n31: n3, k+1) - bv2 * ( vk2' * B(n31: n3, n31: n3, k+1) );
                           end %{if}
                       end %{if}    
                   end % {for k}
                                                     
               end %{if}
               
%                disp('  After one iteration of PQR step in the post-procedure, ... ')
%                A1 = A(:,:,1)
%                pause
                        
            end %{while}
        end %{if}
        
    end %{if}
end %{if}
    
%     disp('  After the post-procedure of reordering ews, ... ')
%     A1 = A(:,:,1)
%     B
%     disp('***********************************************************************')
%     pause

for k = 1: p-1
    Qmat(:, :, k) = Z22(:, :, k);
    Zmat(:, :, k) = Q22(:, :, k+1);    
end    
Qmat(:, :, p) = Z22(:, :, p);
Zmat(:, :, p) = Q22(:, :, 1);

   
% -----------------------------------------------%

    function [L,U,P,Q] = lu_complpiv(A);
    P = []; Q = []; n = size(A,1);
    for k=1:n-1;
      [a,r] = max(abs(A(k:n,k:n)));
      [dummy,c] = max(abs(a));
      cl  = c+k-1;
      rw  = r(c)+k-1;
      A([k,rw],:) = A([rw,k],:);
      A(:,[k,cl]) = A(:,[cl,k]);
      P(k) = rw; Q(k) = cl;
      if A(k,k) ~= 0;
        rs = k+1:n;
        A(rs,k)  = A(rs,k)/A(k,k);
        A(rs,rs) = A(rs,rs)-A(rs,k)*A(k,rs);
      end
    end
    U = tril(A')'; L = tril(A,-1) + eye(n);

