%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Do 2-by-2 eigenvalue subproblems.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

i1 = i + 1;
tmp_b = 1;  bdp = 1;  Emat = eye(2);
for k = p: -1: 1
    tmp_b = B(i, i, k) * B(i1, i1, k);
    bdp = bdp * tmp_b;
    tmp_B = [B(i1, i1, k)  -B(i, i1, k);
                  0         B(i, i, k)];
    tmp_B = tmp_B * A(i: i1, i: i1, k);
    Emat = Emat * tmp_B;
end %{k}  
Emat = Emat ./ bdp;
b = Emat(1, 1) + Emat(2, 2); c = det(Emat);  
if ( sign(b) == 0 )
    ew22 = [sqrt(-c); -sqrt(-c)];
else
    tmp = sqrt(b*b-4*c);
    dn = b + sign(b) * tmp;
    ew22(1, 1) = dn / 2;
    ew22(2, 1) = (2*c) / dn;
end    

if ( imag(ew22(1, 1)) == 0 )  % Two real EWs 
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    %   Periodic QR step with a single shift for 2 by 2 block matrices.
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    maxite = 5;
    
    for ite = 1: maxite
    
        a11 = abs( A(i, i, 1) );
        a22 = abs( A(i1, i1, 1) );
        a21 = abs( A(i1, i, 1) ); 
        err = tol * ( a11 + a22 );
        if ( a21 <= err )
            A(i1, i, 1) = 0;
            break;
        end   
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %    
        %  Evaluate a single shift --- lam.
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        lam = A(i1, i1, 1) * B(i, i, 1);
        b_11 = B(i, i, 1);
        a_11 = 1;
        for k = 2: p
            lam = lam * A(i1, i1, k) * B(i, i, k);
            b_11 = b_11 * B(i, i, k);
            a_11 = a_11 * A(i, i, k);
        end
        lam = lam / bdp ;
        [ss, I22] = min(abs(ew22 - lam)) ;
        lam = ew22(I22, 1) ;
        w2 = A(i: i1, i, 1) ;
        w2(1,1) = w2(1,1) - (lam * b_11) / a_11 ;
        w2 = real(w2) ;
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %
        %  Periodic QR step with single shift 
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        [vk2, bk] = house(w2);
        if ( bk ~= 0 )
            bv2 = bk * vk2;
            A(i: i1, i: n, 1) = A(i: i1, i: n, 1) - bv2 * ( vk2' * A(i: i1, i: n, 1) );
            if ( flag_zmat == 1 )
                Z(i: i1, :, 1) = Z(i: i1, :, 1) - bv2 * ( vk2' * Z(i: i1, :, 1) );  
            end 
            B(i: i1, i: n, 1) = B(i: i1, i: n, 1) - bv2 * ( vk2' * B(i: i1, i: n, 1) );
        end
            
        for k = 1: p
                       
            w2 = [B(i1, i1, k); B(i1, i, k)];
            [vk2, bk] = house(w2);
            if ( bk ~= 0 )
                vk2 = flipud(vk2);
                bv2 = bk * vk2;
                B(1: i1, i: i1, k) = B(1: i1, i: i1, k) - ( B(1: i1, i: i1, k) * vk2 ) * bv2';
                B(i1, i, k) = 0;
                if ( k < p )
                    A(1: i1, i: i1, k+1) = A(1: i1, i: i1, k+1) - ( A(1: i1, i: i1, k+1) * vk2 ) * bv2';
                    if ( flag_qmat == 1 )
                        Q(:, i: i1, k+1) = Q(:, i: i1, k+1) - ( Q(:, i: i1, k+1) * vk2 ) * bv2';  
                    end
                else %{k = p}
                    A(1: i1, i: i1, 1) = A(1: i1, i: i1, 1) - ( A(1: i1, i: i1, 1) * vk2 ) * bv2';
                    %if ( flag_qmat == 1 )
                        Q(:, i: i1, 1) = Q(:, i: i1, 1) - ( Q(:, i: i1, 1) * vk2 ) * bv2';  
                    %end
                end %{if}    
            end  %{if}      
            
            if ( k < p )
                w2 = [A(i, i, k+1); A(i1, i, k+1)];
                [vk2, bk] = house(w2);
                if ( bk ~= 0 )
                    bv2 = bk * vk2;
                    A(i: i1, i: n, k+1) = A(i: i1, i: n, k+1) - bv2 * ( vk2' * A(i: i1, i: n, k+1) );
                    A(i1, i, k+1) = 0;
                    if ( flag_zmat == 1 )
                        Z(i: i1, :, k+1) = Z(i: i1, :, k+1) - bv2 * ( vk2' * Z(i: i1, :, k+1) );  
                    end 
                    B(i: i1, i: n, k+1) = B(i: i1, i: n, k+1) - bv2 * ( vk2' * B(i: i1, i: n, k+1) );
                end %{if}
            end %{if}  
                           
        end % {for k}
    
    end %{ite}    
    
    p_a = ones(2,1);
    p_b = p_a;
    for k = 1: p
        p_a = p_a .* diag(A(i: i1, i: i1, k));
        p_b = p_b .* diag(B(i: i1, i: i1, k));
        alpha_j(k, i: i1) = diag(A(i: i1, i: i1, k))' ;
        beta_j(k, i: i1) = diag(B(i: i1, i: i1, k))' ;
    end   
    Pi_a(i: i1, 1) = p_a;
    Pi_b(i: i1, 1) = p_b;
        
else  % Two complex conjugate EWs.     
    
    Pi_a(i: i1, 1) = ew22;
    Pi_b(i: i1, 1) = ones(2,1);
    alpha_j(:, i: i1) = ones(p, 2);
    alpha_j(1, i: i1) = ew22';
    beta_j(:, i: i1) = ones(p, 2);
    
end %{if}    
